"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const objects_1 = require("../util/objects");
const Validator_1 = __importDefault(require("./Validator"));
const formData_1 = require("../util/formData");
const qs_1 = __importDefault(require("qs"));
const validator = Validator_1.default;
const UNPROCESSABLE_ENTITY = 422;
class BaseProxy {
    constructor(endpoint, parameters) {
        this.endpoint = endpoint;
        this.parameters = parameters;
        this.errors = Validator_1.default;
    }
    get $http() {
        return BaseProxy.$http;
    }
    get $errorsKeyName() {
        return BaseProxy.$errorsKeyName;
    }
    all() {
        return this.submit('get', `/${this.endpoint}`);
    }
    find(id) {
        return this.submit('get', `/${this.endpoint}/${id}`);
    }
    post(payload, config) {
        return this.submit('post', `/${this.endpoint}`, payload, config);
    }
    store(payload, config) {
        return this.post(payload, config);
    }
    put(id, payload) {
        return this.submit('put', `/${this.endpoint}/${id}`, payload);
    }
    putWithFile(id, payload, config) {
        payload._method = 'put';
        return this.submit('post', `/${this.endpoint}/${id}`, payload, config);
    }
    patch(id, payload) {
        return this.submit('patch', `/${this.endpoint}/${id}`, payload);
    }
    delete(id) {
        return this.submit('delete', `/${this.endpoint}/${id}`);
    }
    submit(requestType, url, form, config) {
        const method = BaseProxy.__validateRequestType(requestType);
        this.beforeSubmit();
        return new Promise((resolve, reject) => {
            const data = this.__hasFiles(form) ? formData_1.objectToFormData(form) : form;
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            this.$http[method](this.__getParameterString(url), data, config)
                .then((response) => {
                this.onSuccess();
                const { data } = response;
                resolve(data);
            })
                .catch((error) => {
                this.errors.processing = false;
                validator.processing = false;
                const { response } = error;
                if (response) {
                    const { data, status } = response;
                    if (status === UNPROCESSABLE_ENTITY) {
                        const errors = {};
                        Object.assign(errors, data[this.$errorsKeyName]);
                        this.onFail(errors);
                        validator.fill(errors);
                    }
                    reject(error);
                }
                else {
                    reject(error);
                }
            });
        });
    }
    __getParameterString(url) {
        const query = qs_1.default.stringify(this.parameters, {
            encode: false,
            skipNulls: true,
            addQueryPrefix: true,
        });
        return `${url}${query}`;
    }
    static __validateRequestType(requestType) {
        const requestTypes = [
            'get',
            'delete',
            'head',
            'post',
            'put',
            'patch',
        ];
        if (!requestTypes.includes(requestType)) {
            throw new Error(`\`${requestType}\` is not a valid request type, ` +
                `must be one of: \`${requestTypes.join('`, `')}\`.`);
        }
        return requestType.toLowerCase();
    }
    __hasFiles(form) {
        for (const property in form) {
            if (!form.hasOwnProperty(property)) {
                return false;
            }
            if (typeof window === 'undefined') {
                return false;
            }
            if (this.__hasFilesDeep(form[property])) {
                return true;
            }
        }
        return false;
    }
    __hasFilesDeep(object) {
        if (object === null) {
            return false;
        }
        if (typeof object === 'object') {
            for (const key in object) {
                if (object.hasOwnProperty(key)) {
                    if (objects_1.isFile(object[key])) {
                        return true;
                    }
                }
            }
        }
        if (objects_1.isArray(object)) {
            for (const key in object) {
                if (object.hasOwnProperty(key)) {
                    return this.__hasFilesDeep(object[key]);
                }
            }
        }
        return objects_1.isFile(object);
    }
    setParameters(parameters) {
        Object.keys(parameters).forEach((key) => {
            this.parameters[key] = parameters[key];
        });
        return this;
    }
    setParameter(parameter, value) {
        if (!value) {
            const options = {
                comma: true,
                allowDots: true,
                ignoreQueryPrefix: true,
            };
            const params = qs_1.default.parse(parameter, options);
            return this.setParameters(params);
        }
        this.parameters[parameter] = value;
        return this;
    }
    removeParameters(parameters = []) {
        if (!parameters.length) {
            this.parameters = [];
        }
        else {
            parameters.forEach((parameter) => {
                delete this.parameters[parameter];
            });
        }
        return this;
    }
    removeParameter(parameter) {
        delete this.parameters[parameter];
        return this;
    }
    onFail(errors) {
        this.errors.fill(errors);
    }
    beforeSubmit() {
        if (!this.$http) {
            throw new Error('Vue Api Queries, No http library provided.');
        }
        this.errors.flush();
        this.errors.processing = true;
        this.errors.successful = false;
        validator.flush();
        validator.processing = true;
        validator.successful = false;
    }
    onSuccess() {
        this.errors.processing = false;
        this.errors.successful = true;
        validator.processing = false;
        validator.successful = true;
    }
}
BaseProxy.$errorsKeyName = 'errors';
exports.default = BaseProxy;
